
package cscie160.hw2;
import cscie160.hw2.Floor;
import cscie160.hw2.ElevatorFullException;
/**
 * <p>
 * CSCIE160
 * <p>
 * Assignment 2: Elevator part 2
 * </p>
 * <p>
 * This is a partial simulation of an Elevator by using the
 * sweep algorithm. </p>
 *
 * <p> By David Cheung </p>
 * 
 * @author David Cheung
 */
public class Elevator
{

//constants:
/**
  * The maximum number of people in 
  * an Elevator. This is hardcoded to 10.
  */
	public final int Capacity=10;
/**
  * The number of floors in the building. This is hardcoded to 7.
  */
	public final int numOfFloors=7;
	
//variables

/** 
 * The current floor that the Elevator is on.
 */
	public int currentFloor;
/**
 * The current direction of the elevator: 1=up, -1=down and
 * 0=not moving.
 */
	public int currentDirection; 
	
/** The number of passengers currently in the elevator */
	public int numOfPassengers;
	
/** 
 * Stores the destination requests for each floor and the
 * value would be boolean. Therefore the size of this array
 * should be numOfFloors.
 */
	public boolean [] destRequests;

/**
 * Stores the number of passengers destined for each floor.
 * The size of array should be # of floor, and each
 * value is number of passengers.
 */
	public int [] numOfPassDestForEachFloor;
	
/**
 * Stores the array of floors that the Elevator travels upon.
 */
	public Floor [] arrayOfFloors;
	
//Methods:

/**
 * <p>
 * Moves Elevator from its current floor to the next floor. 
 * This depends on which direction the floor moves. </p>
 * 
 * <p>
 * If the floor is at the top, this will switch the direction
 * of the floor to down (-1) and up (1) if the elevator is
 * at the lowest floor. </p>
 * <p>
 * Calls the stop function if there is a floor that needs to be 
 * stopped. </p>
 * <p>
 * Calls toString function at the end of this.
 *
 * @see Elevator#stop()
 * @see Elevator#toString()
 * 
 */
	public void move()
	{
		currentFloor=currentFloor + currentDirection;
		if (currentFloor == numOfFloors)
		{
			currentDirection=-1;
		}
		else if (currentFloor == 1)
		{
			currentDirection=1;
		}
		if (destRequests[currentFloor] == true || 
			numOfPassDestForEachFloor[currentFloor] >= 1)
		{
			this.stop();
		}
		System.out.println(this);
	}	

/**
 * Prints the current status of the elevator.
 * Called by move each time move is executed.
 * <p>
 * Added for HW 2, this prints the number of people
 * on each of the floors of the elevator.
 *
 * @see Elevator#move()
 */
	public String toString	()
	{
		// Stores the number of passengers string.
		String numOfPassString;		
		String curFloorString;
		String FloorStatusString;
		
		numOfPassString=new String();
		curFloorString= new String();
		FloorStatusString= new String();
		
		numOfPassString="Currently " + numOfPassengers + " on board. \n";
		curFloorString = "Current Floor: "+ currentFloor +" \n" ;

		//For Debug use : Prints out all the floor's passengers waiting.
		for(int i=1;i <=numOfFloors; i++)
		{
			FloorStatusString=FloorStatusString + arrayOfFloors[i].toString() + " ";
		}
		FloorStatusString="Floor Stats: " + FloorStatusString + " \n";
		return numOfPassString + curFloorString + FloorStatusString;
	}
	
/**
 * <p>
 * Does not actually stop the elevator in real time but
 * just to allow the simulation to process the numbers 
 * when stop is called. </p>
 * <p>
 * Now it is changed so that unloadPassengers does the passenger
 * pulling and loading the passengers to the elevator.
 * 
 * <p>
 * Before calling unloadPassenger stop would set the destRequests off for
 * that floor.
 *
 * @see Floor#unloadPassengers(Elevator)
 */
	public void stop()
	{

		destRequests[currentFloor]=false;
		//unload passengers, adding passengers from the floor
		// would be done by floor.unloadpassengers.
		arrayOfFloors[currentFloor].unloadPassengers(this);
	}
	
/**
 * This increments number of passenger destined there by one, and
 * increase the number of passengers in elevator by one.
 * <p>
 * Called by Floor.unloadPassengers
 *
 * @param Floor  Destination floor for a passenger.
 * @see  Floor#unloadPassengers
 */
	public void boardPassenger(int destFloor)
	  throws ElevatorFullException
	{
		 	if(numOfPassengers >= Capacity)
		 	{
		 		throw new ElevatorFullException("Elevator Full!");
		 	}
 	   	numOfPassDestForEachFloor[destFloor]++;
	   	numOfPassengers++;
 	}

/**
 * 
 * Sets the destRequests array.
 * Called by Floor.unloadPassengers.
 *
 * @param numFloor  The floor number of the floor that is
 * making the request.
 * @see Floor#unloadPassengers
 */
 	public void registerRequest(int numFloor)
	 {
	 	destRequests[numFloor]=true;
	 }
 
// Insert constructor

/**
 *	Initializes class by creating an array with size of
 * numOfFloors + 1. Sets initial direction to UP.
 * 
 */
	public Elevator ()
	{
		destRequests = new boolean[numOfFloors+1];
		numOfPassDestForEachFloor = new int[numOfFloors+1];
		//it's easier to keep this convention where array[floor]
		// is consistent, so initializing them would requre 1 more.
		currentDirection=1;
		arrayOfFloors = new Floor[numOfFloors+1];
		for(int i=0; i <= numOfFloors ; i++)
		{
			arrayOfFloors[i]= new Floor(i);
		}
	}
	
/**
 *	Starts the main part of the class, start
 * with adding passengers in the Elevator and then 
 * add a bunch of passengers on some floors. 
 * <p> 
 * Used a FOR loop
 * to do a 2 round trip sweep. Note: I used a FOR loop instead of 
 * WHILE loop because FOR loop allows me to look at
 * the data after the program ends. Whereas WHILE loop will 
 * generate a lot of unnecessary data.
 */

	public static void main(String argv[])
	{
		ElevatorFullException efe = new ElevatorFullException();
		Elevator myElevator;
		myElevator = new Elevator();
		try{
		myElevator.boardPassenger(2);
		myElevator.boardPassenger(3);
		}
		catch(ElevatorFullException ele) {  };
      myElevator.arrayOfFloors[2].loadPassengers(myElevator,14);
      myElevator.arrayOfFloors[5].loadPassengers(myElevator,9);
		for(int i=0;i<37;i++)
		{
			myElevator.move();
		}

	}

}
