
package cscie160.hw3;
import cscie160.hw3.Floor;
import cscie160.hw3.ElevatorFullException;
import cscie160.hw3.Passenger;
import java.util.*;

/**
 * <p>
 * CSCIE160
 * <p>
 * Assignment 3: Elevator part 3
 * </p>
 * <p>
 * This is a partial simulation of an Elevator by using the
 * sweep algorithm. </p>
 *
 * <p> By David Cheung </p>
 * 
 * @author David Cheung
 */
public class Elevator
{

//constants:
/**
  * The maximum number of people in 
  * an Elevator. This is hardcoded to 10.
  */
	private final int Capacity=10;
/**
  * The number of floors in the building. This is initially set to 7.
  */
	private int numOfFloors=7;
	
//variables

/** 
 * The current floor that the Elevator is on.
 */
	private int currentFloor;
/**
 * The current direction of the elevator: 1=up, -1=down and
 * 0=not moving.
 */
	private int currentDirection; 
	
	
/** 
  * Stores the destination requests for each floor and the
  * value would be boolean. Therefore the size of this array
  * should be numOfFloors. This array is used by the passengers
  * in the elevator, whereas the up and down requests are used
  * by the floors that requests stop on up or down.
  * <p>
  * destRequests[] has been replaced by destUpquests and
  * destDownRequests.
  * <p>
  * destUpRequests[]:
  * <p> Stores the destination requests that are going up
  */
	private boolean [] destUpRequests;
	
/**
  * destDownRequests{}:
  * <p> Stores the destination requests that are going down
  */
   private boolean [] destDownRequests;
   
   
/**
 * Stores passengers destined for each floor.
 * The size of array should be # of floor, and each
 * slot is a linklist of passengers.
 * <p>
 * Modified from numOfPassDestForEachFloor, which was
 * an array of int. Now it is an array of linklist.
 * This can be implemented with a hashtable as well,but
 * it seems to me link list works too.
 */
	public LinkedList [] llPassDestForEachFloor;
	
/**
 * Stores the array of floors that the Elevator travels upon.
 */
	public Floor [] arrayOfFloors;
	
//Methods:

/**
 * <p>
 * Moves Elevator from its current floor to the next floor. 
 * This depends on which direction the floor moves. </p>
 * 
 * <p>
 * If the floor is at the top, this will switch the direction
 * of the floor to down (-1) and up (1) if the elevator is
 * at the lowest floor. </p>
 * <p>
 * Calls the stop function if there is a floor that needs to be 
 * stopped. </p>
 * <p>
 * Calls toString function at the end of this.
 *
 * @see Elevator#stop()
 * @see Elevator#toString()
 * 
 */
	public void move()
	{
		currentFloor=currentFloor + currentDirection;

		if (currentFloor == numOfFloors)
		{
			currentDirection=-1;
		}
		else if (currentFloor == 1)
		{
			currentDirection=1;
		}
		if (
				(destUpRequests[currentFloor] == true &&
						currentDirection == 1) || 
				(destDownRequests[currentFloor] == true &&
						currentDirection == -1) ||
				!llPassDestForEachFloor[currentFloor].isEmpty())
		{
			stop();
		}
		System.out.println(this);
	}	

/**
 * Prints the current status of the elevator.
 * Called by move each time move is executed.
 * <p>
 * Added for HW 2, this prints the number of people
 * on each of the floors of the elevator.
 *
 * @see Elevator#move()
 * @obsolete This moved to the building class.
 */
	public String toString	()
	{
		// Stores the number of passengers string.
		String numOfPassString;		
		// Stores Floor Statistic String.
		String curFloorString;
		String FloorStatusString;
		
		numOfPassString=new String();
		curFloorString= new String();
		FloorStatusString= new String();
		
		numOfPassString="Currently " + getNumOfPassengers() + " on board: ";
		
		/** This part shifts through the array of passengers and prints them out*/
		for(int i=1;i<=numOfFloors; i++)
		{
  	       ListIterator iElevator = llPassDestForEachFloor[i].listIterator();
			 while(iElevator.hasNext())
			 {
			 	numOfPassString = numOfPassString + (iElevator.next()).toString();
			 }
		}


		curFloorString = "\nCurrent Floor: "+ currentFloor +" \n" ;

		//For Debug use : Prints out all the floor's passengers waiting.
		for(int i=1;i <=numOfFloors; i++)
		{
			FloorStatusString=FloorStatusString + "Floor: " + i + "\n" + 
			              arrayOfFloors[i].toString();
		}
		FloorStatusString="Floor Stats: \n" + FloorStatusString + " \n";
		return numOfPassString + curFloorString + FloorStatusString;
	}
	
/**
 * <p>
 * Does not actually stop the elevator in real time but
 * just to allow the simulation to process the numbers 
 * when stop is called. </p>
 * <p>
 * Now it is changed so that unloadPassengers does the passenger
 * pulling and loading the passengers to the elevator.
 * 
 * <p>
 * Before calling unloadPassenger stop would set the destRequests off for
 * that floor.
 *
 * @see Floor#unloadPassengers(Elevator)
 */
	public void stop()
	{
		if(currentDirection == 1)	destUpRequests[currentFloor]=false;
		else destDownRequests[currentFloor] = false;
		//unload passengers, adding passengers from the floor
		// would be done by floor.unloadpassengers.
		arrayOfFloors[currentFloor].unloadPassengers(this);
	}
	
/**
 * This increments number of passenger destined there by one, and
 * increase the number of passengers in elevator by one.
 * <p>
 * Called by Floor.unloadPassengers
 *
 * March 14, 2001: Add support for passenger
 * instead of using integer.
 * @param Passenger A passenger to be boarded.
 * @see  Floor#unloadPassengers
 */
	public void boardPassenger(Passenger aPass)
	  throws ElevatorFullException
	{
		 	if(getNumOfPassengers() >= Capacity)
		 	{
		 		throw new ElevatorFullException("Elevator Full!");
		 	}
 	   	llPassDestForEachFloor[aPass.getDestFloor()].add(aPass);
	   	
 	}

/**
 * 
 * Sets the destUpRequests array.
 * Called by Floor.unloadPassengers.
 *
 * @param numFloor  The floor number of the floor that is
 * making the request.
 * @see Floor#unloadPassengers
 */
 	public void registerUpRequest(int numFloor)
	 {
	 	destUpRequests[numFloor]=true;
	 }
/**
 * 
 * Sets the destRequests array.
 * Called by Floor.unloadPassengers.
 *
 * @param numFloor  The floor number of the floor that is
 * making the request.
 * @see Floor#unloadPassengers
 */
 	public void registerDownRequest(int numFloor)
	 {
	 	destDownRequests[numFloor]=true;
	 }

/** The number of passengers currently in the elevator 
  * <p> used to be a data member now it is a method.
  */
	public int getNumOfPassengers()
	{
		int sum = 0;
		for(int i=1;i<=numOfFloors; i++)
		{

			 	sum = sum + llPassDestForEachFloor[i].size();

		}
		return sum;
	}
 
/** 
  * getCurrentFloor
  * gets the current floor of the elevator
  *
  */
  
  public int getCurrentFloor()
  {
  		return currentFloor;
  }
  
/**
  * getCurrentDirection
  * returns the integer value of the direction:
  * 1 = up, -1 = down, 0 = idle.
  */
  public int getCurrentDirection()
  {
  		return currentDirection;
  }
// Insert constructor

/**
 *	Initializes class by creating an array with size of
 * numOfFloors + 1. Sets initial direction to UP.
 * 
 */
	public Elevator (Floor arrFloors[])
	{
		numOfFloors = arrFloors.length - 1;
		destUpRequests = new boolean[numOfFloors+1];
		destDownRequests = new boolean[numOfFloors+1];
		llPassDestForEachFloor = new LinkedList[numOfFloors+1];
		//it's easier to keep this convention where array[floor]
		// is consistent, so initializing them would requre 1 more.
		currentDirection=1;
		
		for(int i=0;i <= numOfFloors; i++)
		{
			llPassDestForEachFloor[i]=new LinkedList();
		}
		
		arrayOfFloors = arrFloors;
/*		arrayOfFloors = new Floor[numOfFloors+1];
		for(int i=0; i <= numOfFloors ; i++)
		{
			arrayOfFloors[i]= new Floor(i);
		}
		//obsolete: this initialization is put on the building.java
*/
	}
	
/**
 *	Starts the main part of the class, start
 * with adding passengers in the Elevator and then 
 * add a bunch of passengers on some floors. 
 * <p> 
 * Used a FOR loop
 * to do a 2 round trip sweep. Note: I used a FOR loop instead of 
 * WHILE loop because FOR loop allows me to look at
 * the data after the program ends. Whereas WHILE loop will 
 * generate a lot of unnecessary data.
 */

	public static void main(String argv[])
	{
		ElevatorFullException efe = new ElevatorFullException();
		Elevator myElevator;
		Floor arrayOfFloors[];
		arrayOfFloors = new Floor[8];
		

		for(int i=0; i < 8 ; i++)
		{
			arrayOfFloors[i]= new Floor(i);
		}

		myElevator = new Elevator(arrayOfFloors);
		try{
		myElevator.boardPassenger(new Passenger(2));
		myElevator.boardPassenger(new Passenger(3));
		}
		catch(ElevatorFullException ele) {  };
		System.out.println(myElevator.arrayOfFloors[1]);

/*      myElevator.arrayOfFloors[2].addOnePassenger(myElevator,new Passenger(2,6));
     myElevator.arrayOfFloors[5].addOnePassenger(myElevator,new Passenger(5,1));
		for(int i=0;i<14;i++)
		{
			myElevator.move();
		}
*/
	}

}
