package cscie160.hw3;
import cscie160.hw3.Elevator;
import cscie160.hw3.ElevatorFullException;
import cscie160.hw3.Passenger;
import cscie160.hw3.Building; //This is for test use....
import java.util.*;
import java.lang.Math;
/**
 * <p>
 * CSCIE160
 * <p>
 * Assignment 2: Elevator part 2
 * </p>
 * <p>
 * Floor class: This class complements the elevator class for the
 * elevator project.
 * </p>
 *
 * <p> By David Cheung </p>
 * 
 * @author David Cheung
 */
public class Floor
{
// Data Members:
/**
 * numOfPassengers:
 * as specified in the HW, numOfPassengers is suppose to be
 * private--ie, Elevator class cannot acces it.
 * <p>
 * However, I used a toString method that will print out 
 * for debuging purposes.
 * <p>
 */
	private int numOfPassengers;
	
/**
 * This is the floor number of the floor.
 * <p>
 * This is used for debugging purposes only.
 */
	private int floorIdentifier;

/** 
  * llResidential
  * <p> This list stores passengers when they
  * arrive at the destinated floor.
  */
  	public LinkedList llResidential;
	  
/**
  * staUp
  * <p> this stack stores the passengers that are in the
  * up queue.
  */
   public LinkedList llUp;
/**
  * llDown
  * <p> this list stores the passengers that are in the
  * down queue.
  */
   public LinkedList llDown;

//Methods

/**
 * unloadPassengers: <p>
 * This Method consists of 2 parts: <p>
 * The first part is from hw1's stop(). The 
 * second part is the new implemented exception
 * feature.
 * <p>
 * After unloading people in the elevator, this
 * function attempts to board passengers to the elevator
 * if sucessful, then method completes.
 * If elevator happens to be full, then this method
 * will cause Elevator's boardPassenger function to 
 * throw an exception. Thus the remaining passengers will
 * be left on the floor. <p>
 * If the exception was thrown, then a registerRequest to the
 * elevator will be initiated. (Done in Catch)
 * <p>
 * March 14, 2001:
 * For HW3, when unloading passengers, a collection of passengers
 * is passed to the floor. This is done to replace the old integer
 * representation of the passengers.
 */
	public void unloadPassengers (Elevator anElevator)
	{
		//this method will unload the passengers from the Elevator and then
		//load the passengers from the floor to the elevator.
		System.out.println("Stopping on floor "+ anElevator.getCurrentFloor());

		/** pass the collection of ppl on that floor to the floor object */
		loadPassengers(anElevator.llPassDestForEachFloor[anElevator.getCurrentFloor()]);
		
		/** Print out # of ppl left */
		System.out.println("Dropped " + 
		  anElevator.llPassDestForEachFloor[anElevator.getCurrentFloor()].size() + 
		  " people off the elevator.");
		
		/** need to clear the list on the Elevator*/
		anElevator.llPassDestForEachFloor[anElevator.getCurrentFloor()].clear();
		
		/**
		 *Now put the ppl that are waiting on the floor into the Elevator.
		 *Assumes all the passengers want to go to the first level.
		 */	
		Passenger tempPass= new Passenger (0); // this is to catch a passenger when it fails.	
		try
		{
			
			if(anElevator.getCurrentDirection() == 1)
			{		
				while(!(llUp.isEmpty()))
				{
					tempPass = (Passenger)llUp.removeFirst();
					anElevator.boardPassenger((Passenger)tempPass);
					System.out.println("loaded one passenger to up elevator");
				}
			}
			if(anElevator.getCurrentDirection() == -1)
			{
				while(!(llDown.isEmpty()))
				{
					tempPass = (Passenger)llDown.removeFirst();
					anElevator.boardPassenger((Passenger)tempPass);					
					System.out.println("loaded one passenger to down elevator");
				}
			}
			
		}
		catch (ElevatorFullException eleFull)
		{
			System.out.println("Elevator Full!...");
			/**Select an elevator request. */
			if(anElevator.getCurrentDirection() == 1)
			{
			 	anElevator.registerUpRequest(anElevator.getCurrentFloor());
				llUp.addFirst((Passenger)tempPass);
			}
			if(anElevator.getCurrentDirection() == -1)
			{
				llDown.addFirst((Passenger)tempPass);
				anElevator.registerDownRequest(anElevator.getCurrentFloor());
			}
		}
	}

/**
  * addOnePassenger
  * <p>
  * This method addes one passenger to the queue. Tries to determine
  * if the passenger needs to go down or go up. Then appropiately add
  * the passenger to the correct list.
  */
  public void addOnePassenger(Elevator myElevator, Passenger aPass)
  {
  		/** If the passenger's destination floor is higher than the
		  * current floor then should queue to the up list, otherwise
		  * down
		  */
		aPass.setCurrFloor(floorIdentifier);
  		if(aPass.getCurrentFloor() < aPass.getDestFloor())
		{
			llUp.add(aPass);
			myElevator.registerUpRequest(floorIdentifier);
		}
		else
		{
			llDown.add(aPass);
			myElevator.registerDownRequest(floorIdentifier);
		}
  }
/** 
 * loadPassengers
 * <p>
 * This method is not called by the elevator, but used as test method
 * to load in passengers. Load one passenger at a time.
 * 
 * <p> for HW3: added random generator for destination
 * floors. Assumes that the max floor is seven, which is 
 * not seen by Floor.
 *
 * @param Elevator		the handle to the elevator
 * @param numOfPeople	state how many people to generate.
 *
 */	
	public void loadPassengers(Elevator myElevator,int numOfPeople)
	{
		double randfloat;
		int targetFloor;
		for(int i=1;i<=numOfPeople;i++)
		{
			randfloat = java.lang.Math.random();
			targetFloor = ((int)(java.lang.Math.round(randfloat * 1000)) % 7) + 1;
			addOnePassenger(myElevator,new Passenger(targetFloor));
		}	
		numOfPassengers=numOfPassengers + numOfPeople;

	
	}

/**
  * loadPassenger
  * <p>
  * This method is called by the elevator. This passes the whole
  * linklist of passengers from elevator's llPassDestForEachFloor and
  * add it to the current floors' residential list.
  *
  */
  public void loadPassengers(LinkedList llGetList)
  {

  		llResidential.addAll(llGetList);	
		  
   	ListIterator itr = llGetList.listIterator();
		while(itr.hasNext())
		{
			((Passenger)itr.next()).arrive();
		}

  }

/** 
 * toString
 * <p>
 * For debugging use. Returns the numOfPassengers for this
 * floor
 */
	public String toString()
	{
		if(llResidential.isEmpty() && llUp.isEmpty() &&
			llDown.isEmpty())
		{
			return "";
		}
		//print residential list:
		String returnString  = new String();
		returnString = "R: ";
   	ListIterator iR = llResidential.listIterator();
		while(iR.hasNext())
		{
			returnString = returnString + (iR.next()).toString();
		}
		//print up list:
		returnString= returnString + "\nUp: ";
   	ListIterator iUp = llUp.listIterator();
		while(iUp.hasNext())
		{
			returnString = returnString + (iUp.next()).toString();
		}
		//print down list:
		returnString= returnString + "\nDown: ";
   	ListIterator iDown = llDown.listIterator();
		while(iDown.hasNext())
		{
			returnString = returnString + (iDown.next()).toString();
		}
		returnString= returnString + "\n";

		return returnString;
	}
	
/**
  * PrintPassengers
  * <p>
  * prints stats for the passengers
  *
  */
  public void PrintPassengers()
  {
  
  		if (numOfPassengers==0)
		  {
		  		System.out.println("No Passengers!");	 
		  }
		else
		  {
		  	 ListIterator iR = llResidential.listIterator();
			 while(iR.hasNext())
			 {
			 	System.out.println(iR.next());
			 }
		  }
  }
  
/**
  * getFloorIdentifier
  * <p>
  * returns the FloorIdentifier of the floor
  *
  */
  public int getFloorIdentifier()
  {
  		return floorIdentifier;
  }
//Constructor

/**
  * Floor constructor:
  * <p>
  * sets floorIdentifier.
  * 
  * @param i The floor number of the floor.
  */	
	public Floor(int i)
	{
		floorIdentifier=i;
		llResidential = new LinkedList();
		llUp = new LinkedList();
		llDown = new LinkedList();
	}
	
}
